@@ -1,8 +1,14 @@
+/*
+ * Blue Engine copyright 2021 © Elham Aryanpur
+ *
+ * The license is same as the one on the root.
+*/
+
 use crate::definitions::{
     Pipeline, Shaders, Textures, UniformBuffer, UniformBuffers, Vertex, VertexBuffers,
 };
 use image::GenericImageView;
-use wgpu::util::DeviceExt;
+use wgpu::{util::DeviceExt, BindGroupLayout};
 
 impl crate::definitions::Renderer {
     /// Creates and adds the pipeline to render queue
@@ -14,7 +20,12 @@ impl crate::definitions::Renderer {
         uniform_index: Option<usize>,
     ) -> Result<usize, anyhow::Error> {
         let pipe = self
-            .build_pipeline(shader_index, vertex_buffer_index, texture_index, uniform_index)
+            .build_pipeline(
+                shader_index,
+                vertex_buffer_index,
+                texture_index,
+                uniform_index,
+            )
             .expect("Couldn't Create Render Pipeline");
         self.render_pipelines.push(pipe);
         Ok(self.render_pipelines.len() - 1)
@@ -60,9 +71,10 @@ impl crate::definitions::Renderer {
         &mut self,
         name: &'static str,
         shader_source: String,
+        uniform_layout: Option<&BindGroupLayout>,
     ) -> Result<usize, anyhow::Error> {
         let shaders = self
-            .build_shaders(name, shader_source)
+            .build_shaders(name, shader_source, uniform_layout)
             .expect("Couldn't create shaders");
         let index = self.shaders.len();
         self.shaders.push(shaders);
@@ -74,6 +86,7 @@ impl crate::definitions::Renderer {
         &mut self,
         name: &str,
         shader_source: String,
+        uniform_layout: Option<&BindGroupLayout>,
     ) -> Result<Shaders, anyhow::Error> {
         let shader = self
             .device
@@ -82,14 +95,17 @@ impl crate::definitions::Renderer {
                 source: wgpu::ShaderSource::Wgsl(shader_source.into()),
                 flags: wgpu::ShaderFlags::all(),
             });
+
+        let mut bind_group_layouts = vec![&self.texture_bind_group_layout, &self.default_uniform_bind_group_layout];
+        if uniform_layout.is_some() {
+            bind_group_layouts.push(uniform_layout.unwrap())
+        }
+
         let render_pipeline_layout =
             self.device
                 .create_pipeline_layout(&wgpu::PipelineLayoutDescriptor {
                     label: Some("Render Pipeline Layout"),
-                    bind_group_layouts: &[
-                        &self.texture_bind_group_layout,
-                        &self.uniform_bind_group_layout,
-                    ],
+                    bind_group_layouts: &bind_group_layouts.as_slice(),
                     push_constant_ranges: &[],
                 });
 
@@ -205,6 +221,11 @@ impl crate::definitions::Renderer {
         Ok(index)
     }
 
+    /// Allows to modify a vertex buffer
+    pub fn get_vertex_buffer(&mut self, index: usize) -> Result<&mut VertexBuffers, anyhow::Error> {
+        Ok(self.vertex_buffers.get_mut(index).unwrap())
+    }
+
     /// Removes vertex and index buffer group
     pub fn remove_vertex_buffer(&mut self, index: usize) -> Result<(), anyhow::Error> {
         self.vertex_buffers.remove(index);
@@ -217,20 +238,20 @@ impl crate::definitions::Renderer {
     pub fn build_and_append_uniform_buffers(
         &mut self,
         uniforms: Vec<UniformBuffer>,
-    ) -> Result<usize, anyhow::Error> {
+    ) -> Result<(usize, BindGroupLayout), anyhow::Error> {
         let uniform_buffers = self
             .build_uniform_buffer(uniforms)
             .expect("Couldn't create uniform buffer");
         let index = self.shaders.len();
-        self.uniform_bind_group.push(uniform_buffers);
-        Ok(index)
+        self.uniform_bind_group.push(uniform_buffers.0);
+        Ok((index, uniform_buffers.1))
     }
 
     /// Creates a new uniform buffer group, according to a list of types
     pub fn build_uniform_buffer(
         &mut self,
         uniforms: Vec<UniformBuffer>,
-    ) -> Result<UniformBuffers, anyhow::Error> {
+    ) -> Result<(UniformBuffers, BindGroupLayout), anyhow::Error> {
         let mut buffer_entry = Vec::<wgpu::BindGroupEntry>::new();
         let mut buffer_layout = Vec::<wgpu::BindGroupLayoutEntry>::new();
         let mut buffer_vec = Vec::<wgpu::Buffer>::new();
@@ -293,15 +314,14 @@ impl crate::definitions::Renderer {
                     label: Some("uniform dynamic bind group layout"),
                     entries: &buffer_layout.as_slice(),
                 });
-        self.uniform_bind_group_layout = uniform_bind_group_layout;
 
         let uniform_bind_group = self.device.create_bind_group(&wgpu::BindGroupDescriptor {
             label: Some("Uniform Bind Groups"),
-            layout: &self.uniform_bind_group_layout,
+            layout: &uniform_bind_group_layout,
             entries: &buffer_entry.as_slice(),
         });
 
-        Ok(uniform_bind_group)
+        Ok((uniform_bind_group, uniform_bind_group_layout))
     }
 
     /// Appends a uniform buffer to render queue
@@ -381,6 +401,7 @@ impl crate::definitions::Renderer {
                 texture: &texture,
                 mip_level: 0,
                 origin: wgpu::Origin3d::ZERO,
+                //aspect: wgpu::TextureAspect::All,
             },
             rgba,
             wgpu::ImageDataLayout {
