@@ -4,9 +4,12 @@
  * The license is same as the one on the root.
 */
 
-use crate::definitions::{Engine, Object, Renderer, WindowDescriptor};
+use crate::definitions::{
+    uniform_type, Camera, Engine, Object, Renderer, UniformBuffer, WindowDescriptor,
+};
+use crate::utils::default_resources::{DEFAULT_COLOR, DEFAULT_SHADER, DEFAULT_TEXTURE};
 use winit::{
-    event::{Event, WindowEvent, *},
+    event::{Event, WindowEvent},
     event_loop::{ControlFlow, EventLoop},
     window::{Window, WindowBuilder},
 };
@@ -42,26 +45,57 @@ impl Engine {
         let window = new_window.build(&event_loop).unwrap();
 
         // The renderer init on current window
-        let renderer = futures::executor::block_on(Renderer::new(&window));
+        let mut renderer = futures::executor::block_on(Renderer::new(&window));
+
+        let camera = Camera::new(&renderer)?;
+
+        let _ = renderer.build_and_append_texture(
+            "Default Texture",
+            Vec::from(DEFAULT_TEXTURE),
+            "clamp",
+        )?;
+
+        let _ = renderer
+            .build_and_append_uniform_buffers(vec![
+                UniformBuffer::Matrix(
+                    "Camera Uniform",
+                    uniform_type::Matrix::from_glm(camera.build_view_projection_matrix()?),
+                ),
+                UniformBuffer::Array(
+                    "Default Color",
+                    uniform_type::Array {
+                        data: DEFAULT_COLOR,
+                    },
+                ),
+            ])?
+            .1;
+
+        let _ = renderer.build_and_append_shaders(
+            "Default Shader",
+            DEFAULT_SHADER.to_string(),
+            None,
+        )?;
 
         Ok(Self {
             window,
             event_loop,
             renderer,
             objects: Vec::new(),
+            camera,
         })
     }
 
     #[allow(unreachable_code)]
     pub fn update_loop<F>(self, mut update_function: F) -> anyhow::Result<()>
     where
-        F: 'static + FnMut(&mut Renderer, &Window, &mut Vec<Object>, &WinitInputHelper),
+        F: 'static + FnMut(&mut Renderer, &Window, &mut Vec<Object>, &WinitInputHelper, &mut Camera),
     {
         let Self {
             event_loop,
             mut renderer,
             window,
             mut objects,
+            mut camera,
         } = self;
 
         // Run the callback of before renderer start
@@ -90,7 +124,7 @@ impl Engine {
                 }
 
                 Event::MainEventsCleared => {
-                    update_function(&mut renderer, &window, &mut objects, &input);
+                    update_function(&mut renderer, &window, &mut objects, &input, &mut camera);
                     objects.iter_mut().for_each(|i| {
                         if i.changed {
                             i.update(&mut renderer, window.inner_size())
